@@ -1,4 +1,4 @@
-/* $Id: fxapi.c,v 1.32 2001/09/23 16:50:01 brianp Exp $ */
+/* $Id: fxapi.c,v 1.35 2002/10/24 23:57:23 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
@@ -508,7 +508,8 @@ fxMesaCreateContext(GLuint win,
       goto errorhandler;
    }
 
-   ctx = fxMesa->glCtx = _mesa_create_context(fxMesa->glVis, shareCtx,	/* share list context */
+   ctx = fxMesa->glCtx = _mesa_create_context(fxMesa->glVis,
+                                              shareCtx,
 					      (void *) fxMesa, GL_TRUE);
    if (!ctx) {
       errorstr = "_mesa_create_context";
@@ -739,7 +740,7 @@ fxMesaSwapBuffers(void)
    }
 
    if (fxMesaCurrentCtx) {
-      _mesa_swapbuffers(fxMesaCurrentCtx->glCtx);
+      _mesa_notifySwapBuffers(fxMesaCurrentCtx->glCtx);
 
       if (fxMesaCurrentCtx->haveDoubleBuffer) {
 
