@@ -1,4 +1,4 @@
-/* $Id: fxdrv.h,v 1.54 2002/10/24 23:57:23 brianp Exp $ */
+/* $Id: fxdrv.h,v 1.55.2.1 2003/01/08 21:33:40 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
@@ -244,7 +244,7 @@ typedef struct
 
    GLboolean alphaTestEnabled;
    GrCmpFnc_t alphaTestFunc;
-   GrAlpha_t alphaTestRefValue;
+   GLfloat alphaTestRefValue;
 
    /* Blend function */
 
@@ -503,7 +503,7 @@ extern void fxDDTexPalette(GLcontext *, struct gl_texture_object *);
 extern void fxDDTexUseGlbPalette(GLcontext *, GLboolean);
 
 extern void fxDDEnable(GLcontext *, GLenum, GLboolean);
-extern void fxDDAlphaFunc(GLcontext *, GLenum, GLchan);
+extern void fxDDAlphaFunc(GLcontext *, GLenum, GLfloat);
 extern void fxDDBlendFunc(GLcontext *, GLenum, GLenum);
 extern void fxDDDepthMask(GLcontext *, GLboolean);
 extern void fxDDDepthFunc(GLcontext *, GLenum);
