@@ -1,10 +1,10 @@
-/* $Id: sparc.c,v 1.5.2.1 2002/04/17 19:08:37 brianp Exp $ */
+/* $Id: sparc.c,v 1.8.4.1 2003/02/08 15:43:27 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  4.0.3
+ * Version:  5.0.1
  * 
- * Copyright (C) 1999-2002  Brian Paul   All Rights Reserved.
+ * Copyright (C) 1999-2003  Brian Paul   All Rights Reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -30,7 +30,6 @@
 
 
 #include "context.h"
-#include "math/m_vertices.h"
 #include "math/m_xform.h"
 #include "tnl/t_context.h"
 #include "sparc.h"
@@ -119,6 +118,8 @@ void _mesa_init_all_sparc_transform_asm(void)
    _mesa_clip_tab[4] = _mesa_sparc_cliptest_points4;
    _mesa_clip_np_tab[4] = _mesa_sparc_cliptest_points4_np;
 #endif
+#if 0
+   /* disable these too.  See bug 673938 */
    _mesa_normal_tab[NORM_TRANSFORM | NORM_NORMALIZE] =
 	   _mesa_sparc_transform_normalize_normals;
    _mesa_normal_tab[NORM_TRANSFORM_NO_ROT | NORM_NORMALIZE] =
@@ -135,6 +136,7 @@ void _mesa_init_all_sparc_transform_asm(void)
 	   _mesa_sparc_normalize_normals;
    _mesa_normal_tab[NORM_RESCALE] =
 	   _mesa_sparc_rescale_normals;
+#endif
 
 #ifdef DEBUG
    _math_test_all_transform_functions("sparc");
@@ -166,12 +168,12 @@ void _mesa_init_sparc_glapi_relocs(void)
 		insn_ptr[2] |= ((disp_addr >> 32) & ((1 << 10) - 1));
 		insn_ptr[3] |= (disp_addr & ((1 << 10) - 1));
 		__glapi_sparc_icache_flush(&insn_ptr[2]);
-		insn_ptr += 10;
+		insn_ptr += 11;
 #else
 		insn_ptr[0] |= (disp_addr >> 10);
 		insn_ptr[1] |= (disp_addr & ((1 << 10) - 1));
 		__glapi_sparc_icache_flush(&insn_ptr[0]);
-		insn_ptr += 4;
+		insn_ptr += 5;
 #endif
 	}
 }
