@@ -1,10 +1,10 @@
-/* $Id: fakeglx.c,v 1.71 2002/10/25 21:06:34 brianp Exp $ */
+/* $Id: fakeglx.c,v 1.75.2.2 2003/03/27 17:57:18 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  4.1
+ * Version:  5.0.1
  *
- * Copyright (C) 1999-2002  Brian Paul   All Rights Reserved.
+ * Copyright (C) 1999-2003  Brian Paul   All Rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -66,7 +66,7 @@
 #define SERVER_MINOR_VERSION 4
 
 /* This is appended onto the glXGetClient/ServerString version strings. */
-#define MESA_GLX_VERSION "Mesa 4.1"
+#define MESA_GLX_VERSION "Mesa 5.0.1"
 
 /* Who implemented this GLX? */
 #define VENDOR "Brian Paul"
@@ -330,6 +330,11 @@ save_glx_visual( Display *dpy, XVisualInfo *vinfo,
                               accumBlueSize, accumAlphaSize, 0, level,
                               GLX_NONE_EXT );
    if (xmvis) {
+      /* Save a copy of the pointer now so we can find this visual again
+       * if we need to search for it in find_glx_visual().
+       */
+      xmvis->vishandle = vinfo;
+      /* add xmvis to the list */
       VisualTable[NumVisuals] = xmvis;
       NumVisuals++;
    }
@@ -417,12 +422,6 @@ find_glx_visual( Display *dpy, XVisualInfo *vinfo )
 {
    int i;
 
-   /* First try to match pointers */
-   for (i=0;i<NumVisuals;i++) {
-      if (VisualTable[i]->display==dpy && VisualTable[i]->vishandle==vinfo) {
-         return VisualTable[i];
-      }
-   }
    /* try to match visual id */
    for (i=0;i<NumVisuals;i++) {
       if (VisualTable[i]->display==dpy
@@ -430,6 +429,14 @@ find_glx_visual( Display *dpy, XVisualInfo *vinfo )
          return VisualTable[i];
       }
    }
+
+   /* if that fails, try to match pointers */
+   for (i=0;i<NumVisuals;i++) {
+      if (VisualTable[i]->display==dpy && VisualTable[i]->vishandle==vinfo) {
+         return VisualTable[i];
+      }
+   }
+
    return NULL;
 }
 
@@ -588,12 +595,8 @@ static XVisualInfo *get_env_visual(Display *dpy, int scr, const char *varname)
       }
    }
 
-   {
-      char s[1000];
-      sprintf(s, "Mesa: GLX unable to find visual class=%s, depth=%d.\n",
-              type, depth );
-      _mesa_warning(NULL, s);
-   }
+   _mesa_warning(NULL, "GLX unable to find visual class=%s, depth=%d.",
+                 type, depth);
 
    return NULL;
 }
@@ -1052,6 +1055,16 @@ static XMesaVisual choose_visual( Display *dpy, int screen, const int *list )
             caveat = *parselist++; /* ignored for now */
             break;
 
+         /*
+          * GLX_ARB_multisample
+          */
+         case GLX_SAMPLE_BUFFERS_ARB:
+            /* ms not supported */
+            return NULL;
+         case GLX_SAMPLES_ARB:
+            /* ms not supported */
+            return NULL;
+
          /*
           * FBConfig attribs.
           */
@@ -1081,7 +1094,8 @@ static XMesaVisual choose_visual( Display *dpy, int screen, const int *list )
 
 	 default:
 	    /* undefined attribute */
-            printf("unexpected attrib 0x%x\n", *parselist);
+            _mesa_warning(NULL, "unexpected attrib 0x%x in choose_visual()",
+                          *parselist);
 	    return NULL;
       }
    }
@@ -1167,8 +1181,18 @@ static XVisualInfo *
 Fake_glXChooseVisual( Display *dpy, int screen, int *list )
 {
    XMesaVisual xmvis = choose_visual(dpy, screen, list);
-   if (xmvis)
+   if (xmvis) {
+#if 0
       return xmvis->vishandle;
+#else
+      /* create a new vishandle - the cached one may be stale */
+      xmvis->vishandle = _mesa_malloc(sizeof(XVisualInfo));
+      if (xmvis->vishandle) {
+         _mesa_memcpy(xmvis->vishandle, xmvis->visinfo, sizeof(XVisualInfo));
+      }
+      return xmvis->vishandle;
+#endif
+   }
    else
       return NULL;
 }
@@ -1625,12 +1649,12 @@ get_config( XMesaVisual xmvis, int attrib, int *value, GLboolean fbconfig )
          return 0;
 
       /*
-       * GLX_EXT_multisample
+       * GLX_ARB_multisample
        */
-      case GLX_SAMPLE_BUFFERS_SGIS:
+      case GLX_SAMPLE_BUFFERS_ARB:
          *value = 0;
          return 0;
-      case GLX_SAMPLES_SGIS:
+      case GLX_SAMPLES_ARB:
          *value = 0;
          return 0;
 
@@ -1663,29 +1687,29 @@ get_config( XMesaVisual xmvis, int attrib, int *value, GLboolean fbconfig )
       case GLX_FBCONFIG_ID_SGIX:
          if (!fbconfig)
             return GLX_BAD_ATTRIBUTE;
-         *value = xmvis->vishandle->visualid;
+         *value = xmvis->visinfo->visualid;
          break;
       case GLX_MAX_PBUFFER_WIDTH:
          if (!fbconfig)
             return GLX_BAD_ATTRIBUTE;
          /* XXX or MAX_WIDTH? */
-         *value = DisplayWidth(xmvis->display, xmvis->vishandle->screen);
+         *value = DisplayWidth(xmvis->display, xmvis->visinfo->screen);
          break;
       case GLX_MAX_PBUFFER_HEIGHT:
          if (!fbconfig)
             return GLX_BAD_ATTRIBUTE;
-         *value = DisplayHeight(xmvis->display, xmvis->vishandle->screen);
+         *value = DisplayHeight(xmvis->display, xmvis->visinfo->screen);
          break;
       case GLX_MAX_PBUFFER_PIXELS:
          if (!fbconfig)
             return GLX_BAD_ATTRIBUTE;
-         *value = DisplayWidth(xmvis->display, xmvis->vishandle->screen) *
-                  DisplayHeight(xmvis->display, xmvis->vishandle->screen);
+         *value = DisplayWidth(xmvis->display, xmvis->visinfo->screen) *
+                  DisplayHeight(xmvis->display, xmvis->visinfo->screen);
          break;
       case GLX_VISUAL_ID:
          if (!fbconfig)
             return GLX_BAD_ATTRIBUTE;
-         *value = xmvis->vishandle->visualid;
+         *value = xmvis->visinfo->visualid;
          break;
 
       default:
@@ -1768,8 +1792,8 @@ static const char *
 Fake_glXQueryServerString( Display *dpy, int screen, int name )
 {
    static char version[1000];
-   sprintf(version, "%d.%d %s", SERVER_MAJOR_VERSION, SERVER_MINOR_VERSION,
-           MESA_GLX_VERSION);
+   _mesa_sprintf(version, "%d.%d %s",
+                 SERVER_MAJOR_VERSION, SERVER_MINOR_VERSION, MESA_GLX_VERSION);
 
    (void) dpy;
    (void) screen;
@@ -1793,8 +1817,8 @@ static const char *
 Fake_glXGetClientString( Display *dpy, int name )
 {
    static char version[1000];
-   sprintf(version, "%d.%d %s", CLIENT_MAJOR_VERSION, CLIENT_MINOR_VERSION,
-           MESA_GLX_VERSION);
+   _mesa_sprintf(version, "%d.%d %s", CLIENT_MAJOR_VERSION,
+                 CLIENT_MINOR_VERSION, MESA_GLX_VERSION);
 
    (void) dpy;
 
@@ -1884,8 +1908,17 @@ static XVisualInfo *
 Fake_glXGetVisualFromFBConfig( Display *dpy, GLXFBConfig config )
 {
    if (dpy && config) {
-      XMesaVisual v = (XMesaVisual) config;
-      return v->vishandle;
+      XMesaVisual xmvis = (XMesaVisual) config;
+#if 0      
+      return xmvis->vishandle;
+#else
+      /* create a new vishandle - the cached one may be stale */
+      xmvis->vishandle = _mesa_malloc(sizeof(XVisualInfo));
+      if (xmvis->vishandle) {
+         _mesa_memcpy(xmvis->vishandle, xmvis->visinfo, sizeof(XVisualInfo));
+      }
+      return xmvis->vishandle;
+#endif
    }
    else {
       return NULL;
@@ -1922,7 +1955,6 @@ Fake_glXCreatePixmap( Display *dpy, GLXFBConfig config, Pixmap pixmap,
                       const int *attribList )
 {
    XMesaVisual v = (XMesaVisual) config;
-   XVisualInfo *visinfo;
    XMesaBuffer b;
 
    (void) dpy;
@@ -1933,17 +1965,6 @@ Fake_glXCreatePixmap( Display *dpy, GLXFBConfig config, Pixmap pixmap,
    if (!dpy || !config || !pixmap)
       return 0;
 
-   visinfo = v->vishandle;
-
-   v = find_glx_visual( dpy, visinfo );
-   if (!v) {
-      v = create_glx_visual( dpy, visinfo );
-      if (!v) {
-         /* unusable visual */
-         return 0;
-      }
-   }
-
    b = XMesaCreatePixmapBuffer( v, pixmap, 0 );
    if (!b) {
       return 0;
@@ -2041,7 +2062,7 @@ Fake_glXQueryDrawable( Display *dpy, GLXDrawable draw, int attribute,
          *value = xmbuf->width * xmbuf->height;
          break;
       case GLX_FBCONFIG_ID:
-         *value = xmbuf->xm_visual->vishandle->visualid;
+         *value = xmbuf->xm_visual->visinfo->visualid;
          return;
       default:
          return;  /* GLX_BAD_ATTRIBUTE? */
@@ -2053,13 +2074,36 @@ static GLXContext
 Fake_glXCreateNewContext( Display *dpy, GLXFBConfig config,
                           int renderType, GLXContext shareList, Bool direct )
 {
-   XMesaVisual v = (XMesaVisual) config;
+   struct fake_glx_context *glxCtx;
+   struct fake_glx_context *shareCtx = (struct fake_glx_context *) shareList;
+   XMesaVisual xmvis = (XMesaVisual) config;
 
    if (!dpy || !config ||
        (renderType != GLX_RGBA_TYPE && renderType != GLX_COLOR_INDEX_TYPE))
       return 0;
 
-   return Fake_glXCreateContext(dpy, v->vishandle, shareList, direct);
+   glxCtx = CALLOC_STRUCT(fake_glx_context);
+   if (!glxCtx)
+      return 0;
+
+   /* deallocate unused windows/buffers */
+   XMesaGarbageCollect();
+
+   glxCtx->xmesaContext = XMesaCreateContext(xmvis,
+                                   shareCtx ? shareCtx->xmesaContext : NULL);
+   if (!glxCtx->xmesaContext) {
+      FREE(glxCtx);
+      return NULL;
+   }
+
+   glxCtx->xmesaContext->direct = GL_FALSE;
+   glxCtx->glxContext.isDirect = GL_FALSE;
+   glxCtx->glxContext.currentDpy = dpy;
+   glxCtx->glxContext.xid = (XID) glxCtx;  /* self pointer */
+
+   assert((void *) glxCtx == (void *) &(glxCtx->glxContext));
+
+   return (GLXContext) glxCtx;
 }
 
 
@@ -2074,7 +2118,7 @@ Fake_glXQueryContext( Display *dpy, GLXContext ctx, int attribute, int *value )
 
    switch (attribute) {
    case GLX_FBCONFIG_ID:
-      *value = xmctx->xm_visual->vishandle->visualid;
+      *value = xmctx->xm_visual->visinfo->visualid;
       break;
    case GLX_RENDER_TYPE:
       if (xmctx->xm_visual->mesa_visual.rgbMode)
@@ -2593,6 +2637,15 @@ Fake_glXFreeMemoryNV( GLvoid *pointer )
 }
 
 
+/*** GLX_MESA_agp_offset */
+
+static GLuint
+Fake_glXGetAGPOffsetMESA( const GLvoid *pointer )
+{
+   (void) pointer;
+   return ~0;
+}
+
 
 extern struct _glxapi_table *_mesa_GetGLXDispatchTable(void);
 struct _glxapi_table *_mesa_GetGLXDispatchTable(void)
@@ -2731,9 +2784,12 @@ struct _glxapi_table *_mesa_GetGLXDispatchTable(void)
    /*** GLX_MESA_set_3dfx_mode ***/
    glx.Set3DfxModeMESA = Fake_glXSet3DfxModeMESA;
 
-   /*** GLX AGP memory allocation ***/
+   /*** GLX_NV_vertex_array_range ***/
    glx.AllocateMemoryNV = Fake_glXAllocateMemoryNV;
    glx.FreeMemoryNV = Fake_glXFreeMemoryNV;
 
+   /*** GLX_MESA_agp_offset ***/
+   glx.GetAGPOffsetMESA = Fake_glXGetAGPOffsetMESA;
+
    return &glx;
 }
