@@ -1,4 +1,4 @@
-/* $Id: context.c,v 1.187 2002/10/29 15:04:35 brianp Exp $ */
+/* $Id: context.c,v 1.188.2.2 2002/12/12 13:02:41 keithw Exp $ */
 
 /*
  * Mesa 3-D graphics library
@@ -49,7 +49,9 @@
 #include "texstate.h"
 #include "mtypes.h"
 #include "varray.h"
+#if FEATURE_NV_vertex_program
 #include "vpstate.h"
+#endif
 #include "vtxfmt.h"
 #include "math/m_translate.h"
 #include "math/m_matrix.h"
@@ -451,7 +453,7 @@ _mesa_initialize_framebuffer( GLframebuffer *buffer,
    assert(buffer);
    assert(visual);
 
-   BZERO(buffer, sizeof(GLframebuffer));
+   _mesa_bzero(buffer, sizeof(GLframebuffer));
 
    /* sanity checks */
    if (softwareDepth ) {
@@ -744,6 +746,8 @@ free_shared_state( GLcontext *ctx, struct gl_shared_state *ss )
    _mesa_DeleteHashTable(ss->VertexPrograms);
 #endif
 
+   _glthread_DESTROY_MUTEX(ss->Mutex);
+
    FREE(ss);
 }
 
