@@ -1,10 +1,10 @@
-/* $Id: mmath.h,v 1.61 2002/10/24 23:57:21 brianp Exp $ */
+/* $Id: mmath.h,v 1.63.2.3 2003/02/05 23:19:58 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  4.1
+ * Version:  5.0.1
  *
- * Copyright (C) 1999-2002 Brian Paul   All Rights Reserved.
+ * Copyright (C) 1999-2003  Brian Paul   All Rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -189,14 +189,18 @@ do {						\
 /*
  * Single precision ceiling, floor, and absolute value functions
  */
-#if defined(__sparc__) /* XXX this probably isn't the ideal test */
+#if defined(__sparc__) || defined(__NeXT__) /* XXX this probably isn't the ideal test */
 #define CEILF(x)   ceil(x)
 #define FLOORF(x)  floor(x)
 #define FABSF(x)   fabs(x)
-#elif defined(__WIN32__)
+#elif defined(__WIN32__) || defined(__IBMC__) || defined(__IBMCPP__)
 #define CEILF(x)   ((GLfloat)ceil(x))
 #define FLOORF(x)  ((GLfloat)floor(x))
 #define FABSF(x)   ((GLfloat)fabs(x))
+#elif defined(XFree86LOADER) && defined(IN_MODULE)
+#define CEILF(x)   ((GLfloat) xf86ceil(x))
+#define FLOORF(x)  ((GLfloat) xf86floor(x))
+#define FABSF(x)   ((GLfloat) xf86fabs(x))
 #else
 #define CEILF(x)   ceilf(x)
 #define FLOORF(x)  floorf(x)
@@ -601,17 +605,25 @@ do {									\
 #define PosFloatToFixed(X)      FloatToFixed(X)
 #define SignedFloatToFixed(X)   FloatToFixed(X)
 
-#ifdef USE_IEEE
 /* Returns TRUE for x == Inf or x == NaN. */
+#ifdef USE_IEEE
 static INLINE int IS_INF_OR_NAN( float x )
 {
    union {float f; int i;} tmp;
    tmp.f = x;
    return !(int)((unsigned int)((tmp.i & 0x7fffffff)-0x7f800000) >> 31);
 }
+#elif defined(isfinite)
+#define IS_INF_OR_NAN(x)        (!isfinite(x))
+#elif defined(finite)
+#define IS_INF_OR_NAN(x)        (!finite(x))
+#elif __VMS
+#define IS_INF_OR_NAN(x)        (!finite(x))
+#elif defined(__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
+#define IS_INF_OR_NAN(x)        (!isfinite(x))
 #else
-#define IS_INF_OR_NAN(x)        (!finite(x)) 
-#endif
+#define IS_INF_OR_NAN(x)        (!finite(x))
+#endif 
 
 
 /*
