@@ -1,10 +1,10 @@
-/* $Id: s_pixeltex.h,v 1.3 2001/03/12 00:48:42 gareth Exp $ */
+/* $Id: s_pixeltex.h,v 1.5 2002/01/27 18:32:03 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  3.5
+ * Version:  4.1
  *
- * Copyright (C) 1999-2001  Brian Paul   All Rights Reserved.
+ * Copyright (C) 1999-2002  Brian Paul   All Rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -31,9 +31,9 @@
 #include "mtypes.h"
 #include "swrast.h"
 
+
 extern void
-_mesa_pixeltexgen(GLcontext *ctx, GLuint n, const GLchan rgba[][4],
-                  GLfloat s[], GLfloat t[], GLfloat r[], GLfloat q[]);
+_swrast_pixel_texture(GLcontext *ctx, struct sw_span *span);
 
 
 #endif
