@@ -1,10 +1,10 @@
-/* $Id: texstore.c,v 1.46 2002/10/28 23:01:24 kschultz Exp $ */
+/* $Id: texstore.c,v 1.47.2.7 2003/03/28 22:01:08 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  4.1
+ * Version:  5.0.1
  *
- * Copyright (C) 1999-2002  Brian Paul   All Rights Reserved.
+ * Copyright (C) 1999-2003  Brian Paul   All Rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -193,7 +193,9 @@ transfer_teximage(GLcontext *ctx, GLuint dimensions,
           texDestFormat == GL_LUMINANCE_ALPHA ||
           texDestFormat == GL_ALPHA ||
           texDestFormat == GL_RGB ||
-          texDestFormat == GL_RGBA);
+          texDestFormat == GL_RGBA ||
+          texDestFormat == GL_COLOR_INDEX ||
+          texDestFormat == GL_DEPTH_COMPONENT);
    ASSERT(texDestAddr);
    ASSERT(srcWidth >= 1);
    ASSERT(srcHeight >= 1);
@@ -295,7 +297,6 @@ transfer_teximage(GLcontext *ctx, GLuint dimensions,
                      + dstYoffset * (dstRowStride / sizeof(GLushort))
                      + dstXoffset * texComponents;
       ASSERT(ctx->Extensions.MESA_ycbcr_texture);
-      printf("copy ycbcr\n");
       for (img = 0; img < srcDepth; img++) {
          GLushort *destRow = dest;
          for (row = 0; row < srcHeight; row++) {
@@ -480,7 +481,9 @@ _mesa_transfer_teximage(GLcontext *ctx, GLuint dimensions,
           baseInternalFormat == GL_LUMINANCE_ALPHA ||
           baseInternalFormat == GL_ALPHA ||
           baseInternalFormat == GL_RGB ||
-          baseInternalFormat == GL_RGBA);
+          baseInternalFormat == GL_RGBA ||
+          baseInternalFormat == GL_COLOR_INDEX ||
+          baseInternalFormat == GL_DEPTH_COMPONENT);
 
    if (transferOps & IMAGE_CONVOLUTION_BIT) {
       _mesa_adjust_image_for_convolution(ctx, dimensions, &postConvWidth,
@@ -979,7 +982,7 @@ _mesa_store_texsubimage1d(GLcontext *ctx, GLenum target, GLint level,
       GLubyte *dest = _mesa_compressed_image_address(xoffset, 0, 0,
                                                      texImage->IntFormat,
                                                      texImage->Width,
-                                                     texImage->Data);
+                                          (GLubyte*) texImage->Data);
       transfer_compressed_teximage(ctx, 1,             /* dimensions */
                                    width, 1, 1,        /* size to replace */
                                    format, type,       /* source format/type */
@@ -1028,7 +1031,7 @@ _mesa_store_texsubimage2d(GLcontext *ctx, GLenum target, GLint level,
       GLubyte *dest = _mesa_compressed_image_address(xoffset, yoffset, 0,
                                                      texImage->IntFormat,
                                                      texImage->Width,
-                                                     texImage->Data);
+                                          (GLubyte*) texImage->Data);
       transfer_compressed_teximage(ctx, 2,             /* dimensions */
                                    width, height, 1,   /* size to replace */
                                    format, type,       /* source format/type */
@@ -1076,7 +1079,7 @@ _mesa_store_texsubimage3d(GLcontext *ctx, GLenum target, GLint level,
       GLubyte *dest = _mesa_compressed_image_address(xoffset, yoffset, zoffset,
                                                      texImage->IntFormat,
                                                      texImage->Width,
-                                                     texImage->Data);
+                                          (GLubyte*) texImage->Data);
       transfer_compressed_teximage(ctx, 3,              /* dimensions */
                                    width, height, depth,/* size to replace */
                                    format, type,       /* source format/type */
@@ -1230,7 +1233,8 @@ _mesa_store_compressed_texsubimage2d(GLcontext *ctx, GLenum target,
                                                texImage->Width);
    dest = _mesa_compressed_image_address(xoffset, yoffset, 0,
                                          texImage->IntFormat,
-                                         texImage->Width, texImage->Data);
+                                         texImage->Width,
+                              (GLubyte*) texImage->Data);
 
    bytesPerRow = srcRowStride;
    rows = height / 4;
@@ -1895,8 +1899,8 @@ _mesa_generate_mipmap(GLcontext *ctx, GLenum target,
 {
    const struct gl_texture_image *srcImage;
    const struct gl_texture_format *convertFormat;
-   const GLubyte *srcData;
-   GLubyte *dstData;
+   const GLubyte *srcData = NULL;
+   GLubyte *dstData = NULL;
    GLint level, maxLevels;
 
    ASSERT(texObj);
@@ -1932,12 +1936,12 @@ _mesa_generate_mipmap(GLcontext *ctx, GLenum target,
       size = _mesa_bytes_per_pixel(srcImage->Format, CHAN_TYPE)
          * srcImage->Width * srcImage->Height * srcImage->Depth + 20;
       /* 20 extra bytes, just be safe when calling last FetchTexel */
-      srcData = MALLOC(size);
+      srcData = (GLubyte *) MALLOC(size);
       if (!srcData) {
          _mesa_error(ctx, GL_OUT_OF_MEMORY, "generate mipmaps");
          return;
       }
-      dstData = MALLOC(size / 2);  /* 1/4 would probably be OK */
+      dstData = (GLubyte *) MALLOC(size / 2);  /* 1/4 would probably be OK */
       if (!dstData) {
          _mesa_error(ctx, GL_OUT_OF_MEMORY, "generate mipmaps");
          FREE((void *) srcData);
@@ -1969,7 +1973,7 @@ _mesa_generate_mipmap(GLcontext *ctx, GLenum target,
       GLint border, bytesPerTexel;
 
       /* get src image parameters */
-      srcImage = texObj->Image[level];
+      srcImage = _mesa_select_tex_image(ctx, texUnit, target, level);
       ASSERT(srcImage);
       srcWidth = srcImage->Width;
       srcHeight = srcImage->Height;
@@ -2101,10 +2105,10 @@ _mesa_generate_mipmap(GLcontext *ctx, GLenum target,
          _mesa_compress_teximage(ctx,
                                  dstWidth, dstHeight, /* size */
                                  srcFormat,           /* source format */
-                                 dstData,             /* source buffer */
+                (const GLchan *) dstData,             /* source buffer */
                                  dstWidth,            /* source row stride */
                                  dstImage->TexFormat, /* dest format */
-                                 dstImage->Data,      /* dest buffer */
+                      (GLubyte*) dstImage->Data,      /* dest buffer */
                                  dstRowStride );      /* dest row stride */
 
          /* swap src and dest pointers */
