@@ -1,10 +1,10 @@
-/* $Id: vpexec.c,v 1.18 2002/10/25 21:06:31 brianp Exp $ */
+/* $Id: vpexec.c,v 1.19.2.3 2003/03/29 15:35:40 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  4.1
+ * Version:  5.0.1
  *
- * Copyright (C) 1999-2002  Brian Paul   All Rights Reserved.
+ * Copyright (C) 1999-2003  Brian Paul   All Rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -82,6 +82,7 @@
 #include "macros.h"
 #include "mtypes.h"
 #include "vpexec.h"
+#include "mmath.h"
 #include "math/m_matrix.h"
 
 
@@ -276,11 +277,11 @@ fetch_vector4( const struct vp_src_register *source,
    const GLfloat *src;
 
    if (source->RelAddr) {
-      GLint reg = source->Register + machine->AddressReg;
-      if (reg < VP_PROG_REG_START || reg > VP_PROG_REG_END)
+      const GLint reg = source->Register + machine->AddressReg;
+      if (reg < 0 || reg > VP_NUM_PROG_REGS)
          src = zero;
       else
-         src = machine->Registers[reg];
+         src = machine->Registers[VP_PROG_REG_START + reg];
    }
    else {
       src = machine->Registers[source->Register];
@@ -313,11 +314,11 @@ fetch_vector1( const struct vp_src_register *source,
    const GLfloat *src;
 
    if (source->RelAddr) {
-      GLint reg = source->Register + machine->AddressReg;
-      if (reg < VP_PROG_REG_START || reg > VP_PROG_REG_END)
+      const GLint reg = source->Register + machine->AddressReg;
+      if (reg < 0 || reg > VP_NUM_PROG_REGS)
          src = zero;
       else
-         src = machine->Registers[reg];
+         src = machine->Registers[VP_PROG_REG_START + reg];
    }
    else {
       src = machine->Registers[source->Register];
@@ -352,17 +353,6 @@ store_vector4( const struct vp_dst_register *dest, struct vp_machine *machine,
 }
 
 
-/**
- * Compute log base two of x.
- *
- * if log(b,x) is log base b of x then
- *   log(b,x) = log(n,x) / log(n,b)
- * therefore, log(2,x) = log(e,x) / log(e,2).
- *
- * log(e,2) = ln(2) = 0.69314718
- */
-#define LOG2(x)  (log(x) * (1.0 / 0.69314718))
-
 /**
  * Set x to positive or negative infinity.
  */
@@ -448,15 +438,11 @@ _mesa_exec_program(GLcontext *ctx, const struct vp_program *program)
                floor_t0 = (float) floor(t[0]);
                if (floor_t0 > FLT_MAX_EXP) {
                   SET_POS_INFINITY(q[0]);
-                  q[1] = 0.0F;
                   SET_POS_INFINITY(q[2]);
-                  q[3] = 1.0F;
                }
                else if (floor_t0 < FLT_MIN_EXP) {
                   q[0] = 0.0F;
-                  q[1] = 0.0F;
                   q[2] = 0.0F;
-                  q[3] = 0.0F;
                }
                else {
 #ifdef USE_IEEE
@@ -467,11 +453,11 @@ _mesa_exec_program(GLcontext *ctx, const struct vp_program *program)
 #else
                   q[0] = (GLfloat) pow(2.0, floor_t0);
 #endif
-                  q[1] = t[0] - floor_t0;
                   q[2] = (GLfloat) (q[0] * LOG2(q[1]));
-                  q[3] = 1.0F;
                }
-               store_vector4( &inst->DstReg, machine, t );
+               q[1] = t[0] - floor_t0;
+               q[3] = 1.0F;
+               store_vector4( &inst->DstReg, machine, q );
             }
             break;
          case LOG:
@@ -487,7 +473,7 @@ _mesa_exec_program(GLcontext *ctx, const struct vp_program *program)
 #ifdef VMS
                   if (abs_t0 == __MAXFLOAT) {
 #else
-                  if (!finite(abs_t0)) {
+                  if (IS_INF_OR_NAN(abs_t0)) {
 #endif
                      SET_POS_INFINITY(q[0]);
                      q[1] = 1.0F;
