@@ -1,10 +1,10 @@
-/* $Id: vpstate.c,v 1.12 2002/10/25 21:06:33 brianp Exp $ */
+/* $Id: vpstate.c,v 1.12.2.2 2003/03/21 13:24:52 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  4.1
+ * Version:  5.0.1
  *
- * Copyright (C) 1999-2001  Brian Paul   All Rights Reserved.
+ * Copyright (C) 1999-2003  Brian Paul   All Rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -138,20 +138,26 @@ void _mesa_BindProgramNV(GLenum target, GLuint id)
    /* NOTE: binding to a non-existant program is not an error.
     * That's supposed to be caught in glBegin.
     */
-   vprog = (struct vp_program *)
-      _mesa_HashLookup(ctx->Shared->VertexPrograms, id);
+   if (id == 0) {
+      /* OK, the null program object */
+      vprog = NULL;
+   }
+   else {
+      vprog = (struct vp_program *)
+         _mesa_HashLookup(ctx->Shared->VertexPrograms, id);
 
-   if (!vprog && id > 0){
-      /* new program ID */
-      vprog = CALLOC_STRUCT(vp_program);
-      if (!vprog) {
-         _mesa_error(ctx, GL_OUT_OF_MEMORY, "glBindProgramNV");
-         return;
+      if (!vprog && id > 0){
+         /* new program ID */
+         vprog = CALLOC_STRUCT(vp_program);
+         if (!vprog) {
+            _mesa_error(ctx, GL_OUT_OF_MEMORY, "glBindProgramNV");
+            return;
+         }
+         vprog->Target = target;
+         vprog->Resident = GL_TRUE;
+         vprog->RefCount = 1;
+         _mesa_HashInsert(ctx->Shared->VertexPrograms, id, vprog);
       }
-      vprog->Target = target;
-      vprog->Resident = GL_TRUE;
-      vprog->RefCount = 1;
-      _mesa_HashInsert(ctx->Shared->VertexPrograms, id, vprog);
    }
 
    ctx->VertexProgram.CurrentID = id;
@@ -273,8 +279,8 @@ void _mesa_GenProgramsNV(GLsizei n, GLuint *ids)
 GLboolean _mesa_AreProgramsResidentNV(GLsizei n, const GLuint *ids,
                                       GLboolean *residences)
 {
-   GLint i;
-   GLboolean retVal;
+   GLint i, j;
+   GLboolean allResident = GL_TRUE;
    GET_CURRENT_CONTEXT(ctx);
    ASSERT_OUTSIDE_BEGIN_END_WITH_RETVAL(ctx, GL_FALSE);
 
@@ -283,30 +289,33 @@ GLboolean _mesa_AreProgramsResidentNV(GLsizei n, const GLuint *ids,
       return GL_FALSE;
    }
 
-   retVal = GL_TRUE;
-
    for (i = 0; i < n; i++) {
-      struct vp_program *vprog;
-
+      const struct vp_program *prog;
       if (ids[i] == 0) {
-         _mesa_error(ctx, GL_INVALID_VALUE, "glAreProgramsResidentNV(id)");
+         _mesa_error(ctx, GL_INVALID_VALUE, "glAreProgramsResidentNV");
          return GL_FALSE;
       }
-
-      vprog = (struct vp_program *)
+      prog = (const struct vp_program *)
          _mesa_HashLookup(ctx->Shared->VertexPrograms, ids[i]);
-
-      if (!vprog) {
-         _mesa_error(ctx, GL_INVALID_VALUE, "glAreProgramsResidentNV(id)");
+      if (!prog) {
+         _mesa_error(ctx, GL_INVALID_VALUE, "glAreProgramsResidentNV");
          return GL_FALSE;
       }
-
-      *residences = vprog->Resident;
-      if (!vprog->Resident)
-         retVal = GL_FALSE;
+      if (prog->Resident) {
+	 if (!allResident)
+	    residences[i] = GL_TRUE;
+      }
+      else {
+         if (allResident) {
+	    allResident = GL_FALSE;
+	    for (j = 0; j < i; j++)
+	       residences[j] = GL_TRUE;
+	 }
+	 residences[i] = GL_FALSE;
+      }
    }
 
-   return retVal;
+   return allResident;
 }
 
 
